from enum import Enum


class W24Icon(str, Enum):
    """List of all Icons that are supported by the SheetPorter."""

    PROJECTION_METHOD_1 = "PROJECTION_METHOD_1"
    PROJECTION_METHOD_3 = "PROJECTION_METHOD_3"
